package p;

import atomicsets.OrderedLock;

/*
 * @(#)LinkedList.java	1.61 04/02/19
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Linked list implementation of the <tt>List</tt> interface.  Implements all
 * optional list operations, and permits all elements (including
 * <tt>null</tt>).  In addition to implementing the <tt>List</tt> interface,
 * the <tt>LinkedList</tt> class provides uniformly named methods to
 * <tt>get</tt>, <tt>remove</tt> and <tt>insert</tt> an element at the
 * beginning and end of the list.  These operations allow linked lists to be
 * used as a stack, queue, or double-ended queue (deque).<p>
 * 
 * The class implements the <tt>Queue</tt> interface, providing
 * first-in-first-out queue operations for <tt>add</tt>,
 * <tt>poll</tt>, etc. Other stack and deque operations could be
 * easily recast in terms of the standard list operations.  They're
 * included here primarily for convenience, though they may run
 * slightly faster than the equivalent List operations.<p>
 *
 * All of the operations perform as could be expected for a doubly-linked
 * list.  Operations that index into the list will traverse the list from
 * the beginning or the end, whichever is closer to the specified index.<p>
 *
 * <b>Note that this implementation is not synchronized.</b> If multiple
 * threads access a list concurrently, and at least one of the threads
 * modifies the list structurally, it <i>must</i> be synchronized
 * externally.  (A structural modification is any operation that adds or
 * deletes one or more elements; merely setting the value of an element is not
 * a structural modification.)  This is typically accomplished by
 * synchronizing on some object that naturally encapsulates the list.  If no
 * such object exists, the list should be "wrapped" using the
 * Collections.synchronizedList method.  This is best done at creation time,
 * to prevent accidental unsynchronized access to the list: <pre>
 *     List list = Collections.synchronizedList(new LinkedList(...));
 * </pre><p>
 * 
 * The iterators returned by the this class's <tt>iterator</tt> and
 * <tt>listIterator</tt> methods are <i>fail-fast</i>: if the list is
 * structurally modified at any time after the iterator is created, in any way
 * except through the Iterator's own <tt>remove</tt> or <tt>add</tt> methods,
 * the iterator will throw a <tt>ConcurrentModificationException</tt>.  Thus,
 * in the face of concurrent modification, the iterator fails quickly and
 * cleanly, rather than risking arbitrary, non-deterministic behavior at an
 * undetermined time in the future.
 *
 * <p>Note that the fail-fast behavior of an iterator cannot be guaranteed
 * as it is, generally speaking, impossible to make any hard guarantees in the
 * presence of unsynchronized concurrent modification.  Fail-fast iterators
 * throw <tt>ConcurrentModificationException</tt> on a best-effort basis.
 * Therefore, it would be wrong to write a program that depended on this
 * exception for its correctness:   <i>the fail-fast behavior of iterators
 * should be used only to detect bugs.</i><p>
 *
 * This class is a member of the
 * <a href="{@docRoot}/../guide/collections/index.html">
 * Java Collections Framework</a>.
 *
 * @author  Josh Bloch
 * @version 1.61, 02/19/04
 * @see	    List
 * @see	    ArrayList
 * @see	    Vector
 * @see	    Collections#synchronizedList(List)
 * @since 1.2
 * @param <E> the type of elements held in this collection
 */
 
public class LinkedList
    extends AbstractSequentialList
    implements List, Queue, Cloneable, java.io.Serializable
{
    /*atomic(L)*/ private transient LinkedList_Entry header/*E=this.L*/ = new LinkedList_Entry/*E=this.L*/(null, null, null, this.$lock_L);
    /*atomic(L)*/ private transient int size = 0;

    public int getSize_internal(){
    	return size;
    }

	public int getSize() {
		synchronized (this.$lock_L) {
			return size;
		}
	}
    
    /**
	 * Constructs an empty list.
	 */
	public LinkedList() {
		this(new OrderedLock());
	}

	/**
     * Constructs an empty list.
     */
    public LinkedList(OrderedLock L) {
        super(L);
		header.next = header.previous = header; 
    }

    /**
	 * Constructs a list containing the elements of the specified collection, in the order they are returned by the collection's iterator.
	 * @param c  the collection whose elements are to be placed into this list.
	 * @throws NullPointerException  if the specified collection is null.
	 */
	public LinkedList(Collection c) {
		this(c, new OrderedLock());
	}

	/**
     * Constructs a list containing the elements of the specified
     * collection, in the order they are returned by the collection's
     * iterator.
     *
     * @param  c the collection whose elements are to be placed into this list.
     * @throws NullPointerException if the specified collection is null.
     */
     public LinkedList(/*unitfor(L)*/ Collection c, OrderedLock L) {
		this(L);
		synchronized (c.getLockForL()) {
		 addAll_internal(c);
		 }
	}

    /** 
     * Returns the first element in this list.
     *
     * @return the first element in this list.
     * @throws    NoSuchElementException if this list is empty.
     */
    public Object getFirst_internal() {
	if (size==0)
	    throw new NoSuchElementException();

	return header.next.element;
    }

	/**
	 * Returns the first element in this list.
	 * @return  the first element in this list.
	 * @throws NoSuchElementException  if this list is empty.
	 */
	public Object getFirst() {
		synchronized (this.$lock_L) {
			if (size==0)
			    throw new NoSuchElementException();
			return header.next.element;
		}
	} 
    
 
    
    /**
     * Returns the last element in this list.
     *
     * @return the last element in this list.
     * @throws    NoSuchElementException if this list is empty.
     */
    public Object getLast_internal()  {
	if (size==0)
	    throw new NoSuchElementException();

	return header.previous.element;
    }

	/**
	 * Returns the last element in this list.
	 * @return  the last element in this list.
	 * @throws NoSuchElementException  if this list is empty.
	 */
	public Object getLast() {
		synchronized (this.$lock_L) {
			if (size==0)
			    throw new NoSuchElementException();
			return header.previous.element;
		}
	} 

    /**
     * Removes and returns the first element from this list.
     *
     * @return the first element from this list.
     * @throws    NoSuchElementException if this list is empty.
     */
    public Object removeFirst_internal() {
	return remove_internal(header.next);
    }

	/**
	 * Removes and returns the first element from this list.
	 * @return  the first element from this list.
	 * @throws NoSuchElementException  if this list is empty.
	 */
	public Object removeFirst() {
		synchronized (this.$lock_L) {
			return remove_internal(header.next);
		}
	}
 
    /**
     * Removes and returns the last element from this list.
     *
     * @return the last element from this list.
     * @throws    NoSuchElementException if this list is empty.
     */
    public Object removeLast_internal() {
	return remove_internal(header.previous);
    }

	/**
	 * Removes and returns the last element from this list.
	 * @return  the last element from this list.
	 * @throws NoSuchElementException  if this list is empty.
	 */
	public Object removeLast() {
		synchronized (this.$lock_L) {
			return remove_internal(header.previous);
		}
	}

    /**
     * Inserts the given element at the beginning of this list.
     *
     * @param o the element to be inserted at the beginning of this list.
     */
    public void addFirst_internal(Object o) {
	addBefore_internal(o, header.next);
    }

	/**
	 * Inserts the given element at the beginning of this list.
	 * @param o  the element to be inserted at the beginning of this list.
	 */
	public void addFirst(Object o) {
		synchronized (this.$lock_L) {
			addBefore_internal(o, header.next);
		}
	}

    /**
     * Appends the given element to the end of this list.  (Identical in
     * function to the <tt>add</tt> method; included only for consistency.)
     *
     * @param o the element to be inserted at the end of this list.
     */
    public void addLast_internal(Object o) {
	addBefore_internal(o, header);
    }

	/**
	 * Appends the given element to the end of this list.  (Identical in function to the <tt>add</tt> method; included only for consistency.)
	 * @param o  the element to be inserted at the end of this list.
	 */
	public void addLast(Object o) {
		synchronized (this.$lock_L) {
			addBefore_internal(o, header);
		}
	}

    /**
     * Returns <tt>true</tt> if this list contains the specified element.
     * More formally, returns <tt>true</tt> if and only if this list contains
     * at least one element <tt>e</tt> such that <tt>(o==null ? e==null
     * : o.equals(e))</tt>.
     *
     * @param o element whose presence in this list is to be tested.
     * @return <tt>true</tt> if this list contains the specified element.
     */
    public boolean contains_internal(Object o) {
        return indexOf_internal(o) != -1;
    }

	/**
	 * Returns <tt>true</tt> if this list contains the specified element. More formally, returns <tt>true</tt> if and only if this list contains at least one element <tt>e</tt> such that <tt>(o==null ? e==null : o.equals(e))</tt>.
	 * @param o  element whose presence in this list is to be tested.
	 * @return  <tt>true</tt> if this list contains the specified element.
	 */
	public boolean contains(Object o) {
		synchronized (this.$lock_L) {
			return indexOf_internal(o) != -1;
		}
	}

    /**
     * Returns the number of elements in this list.
     *
     * @return the number of elements in this list.
     */
    public int size_internal() {
	return size;
    }

	/**
	 * Returns the number of elements in this list.
	 * @return  the number of elements in this list.
	 */
	public int size() {
		synchronized (this.$lock_L) {
			return size;
		}
	}

    /**
     * Appends the specified element to the end of this list.
     *
     * @param o element to be appended to this list.
     * @return <tt>true</tt> (as per the general contract of
     * <tt>Collection.add</tt>).
     */
    public boolean add_internal(Object o) {
	addBefore_internal(o, header);
        return true;
    }

	/**
	 * Appends the specified element to the end of this list.
	 * @param o  element to be appended to this list.
	 * @return  <tt>true</tt> (as per the general contract of <tt>Collection.add</tt>).
	 */
	public boolean add(Object o) {
		synchronized (this.$lock_L) {
			addBefore_internal(o, header);
		}
		return true;
	}

    /**
     * Removes the first occurrence of the specified element in this list.  If
     * the list does not contain the element, it is unchanged.  More formally,
     * removes the element with the lowest index <tt>i</tt> such that
     * <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt> (if such an
     * element exists).
     *
     * @param o element to be removed from this list, if present.
     * @return <tt>true</tt> if the list contained the specified element.
     */
    public boolean remove_internal(Object o) {
        if (o==null) {
            for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
                if (e.element==null) {
                    remove_internal(e);
                    return true;
                }
            }
        } else {
            for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
                if (o.equals(e.element)) {
                    remove_internal(e);
                    return true;
                }
            }
        }
        return false;
    }

	/**
	 * Removes the first occurrence of the specified element in this list.  If the list does not contain the element, it is unchanged.  More formally, removes the element with the lowest index <tt>i</tt> such that <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt> (if such an element exists).
	 * @param o  element to be removed from this list, if present.
	 * @return  <tt>true</tt> if the list contained the specified element.
	 */
	public boolean remove(Object o) {
		synchronized (this.$lock_L) {
			if (o==null) {
			    for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
			        if (e.element==null) {
			            remove_internal(e);
			            return true;
			        }
			    }
			} else {
			    for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
			        if (o.equals(e.element)) {
			            remove_internal(e);
			            return true;
			        }
			    }
			}
		}
		return false;
	}

    /**
     * Appends all of the elements in the specified collection to the end of
     * this list, in the order that they are returned by the specified
     * collection's iterator.  The behavior of this operation is undefined if
     * the specified collection is modified while the operation is in
     * progress.  (This implies that the behavior of this call is undefined if
     * the specified Collection is this list, and this list is nonempty.)
     *
     * @param c the elements to be inserted into this list.
     * @return <tt>true</tt> if this list changed as a result of the call.
     * @throws NullPointerException if the specified collection is null.
     */
    public boolean addAll_internal(/*unitfor(L)*/ Collection c) {
        return addAll_internal(size, c);
    }

	/**
	 * Appends all of the elements in the specified collection to the end of this list, in the order that they are returned by the specified collection's iterator.  The behavior of this operation is undefined if the specified collection is modified while the operation is in progress.  (This implies that the behavior of this call is undefined if the specified Collection is this list, and this list is nonempty.)
	 * @param c  the elements to be inserted into this list.
	 * @return  <tt>true</tt> if this list changed as a result of the call.
	 * @throws NullPointerException  if the specified collection is null.
	 */
	public boolean addAll(Collection c) {
		OrderedLock l1 = null, l2 = null;
		{
			OrderedLock l3 = c.getLockForL();
			OrderedLock l4 = this.$lock_L;
			if (l3.getIndex() > l4.getIndex()) {
				l1 = l3;
				l2 = l4;
			} else {
				l1 = l4;
				l2 = l3;
			}
		}
		synchronized (l1) {
			synchronized (l2) {
				return addAll_internal(size, c);
			}
		}
	}

    /**
     * Inserts all of the elements in the specified collection into this
     * list, starting at the specified position.  Shifts the element
     * currently at that position (if any) and any subsequent elements to
     * the right (increases their indices).  The new elements will appear
     * in the list in the order that they are returned by the
     * specified collection's iterator.
     *
     * @param index index at which to insert first element
     *		    from the specified collection.
     * @param c elements to be inserted into this list.
     * @return <tt>true</tt> if this list changed as a result of the call.
     * @throws IndexOutOfBoundsException if the specified index is out of
     *            range (<tt>index &lt; 0 || index &gt; size()</tt>).
     * @throws NullPointerException if the specified collection is null.
     */
    public boolean addAll_internal(int index, /*unitfor(L)*/ Collection c) {
        if (index < 0 || index > size)
            throw new IndexOutOfBoundsException("Index: "+index+
                                                ", Size: "+size);
        Object[] a = c.toArray_internal();
        int numNew = a.length;
        if (numNew==0)
            return false;
	incrementModCount_internal();

        LinkedList_Entry successor/*E=this.L*/ = (index==size ? header : entry_internal(index));
        LinkedList_Entry predecessor/*E=this.L*/ = successor.previous;
	for (int i=0; i<numNew; i++) {
            LinkedList_Entry e/*E=this.L*/ = new LinkedList_Entry/*E=this.L*/((Object)a[i], successor, predecessor, this.$lock_L);
            predecessor.next = e;
            predecessor = e;
        }
        successor.previous = predecessor;

        size += numNew;
        return true;
    }

	/**
	 * Inserts all of the elements in the specified collection into this list, starting at the specified position.  Shifts the element currently at that position (if any) and any subsequent elements to the right (increases their indices).  The new elements will appear in the list in the order that they are returned by the specified collection's iterator.
	 * @param index  index at which to insert first element from the specified collection.
	 * @param c  elements to be inserted into this list.
	 * @return  <tt>true</tt> if this list changed as a result of the call.
	 * @throws IndexOutOfBoundsException  if the specified index is out of range (<tt>index &lt; 0 || index &gt; size()</tt>).
	 * @throws NullPointerException  if the specified collection is null.
	 */
	public boolean addAll(int index, Collection c) {
		OrderedLock l1 = null, l2 = null;
		{
			OrderedLock l3 = c.getLockForL();
			OrderedLock l4 = this.$lock_L;
			if (l3.getIndex() > l4.getIndex()) {
				l1 = l3;
				l2 = l4;
			} else {
				l1 = l4;
				l2 = l3;
			}
		}
		synchronized (l1) {
			synchronized (l2) {
				if (index < 0 || index > size)
				    throw new IndexOutOfBoundsException("Index: "+index+
				                                        ", Size: "+size);
				Object[] a = c.toArray_internal();
				int numNew = a.length;
				if (numNew==0)
				    return false;
				incrementModCount_internal();
				LinkedList_Entry successor/*E=this.L*/ = (index==size ? header : entry_internal(index));
				LinkedList_Entry predecessor/*E=this.L*/ = successor.previous;
				for (int i=0; i<numNew; i++) {
				        LinkedList_Entry e/*E=this.L*/ = new LinkedList_Entry/*E=this.L*/((Object)a[i], successor, predecessor, this.$lock_L);
				        predecessor.next = e;
				        predecessor = e;
				    }
				successor.previous = predecessor;
				size += numNew;
			}
		}
		return true;
	}

    /**
     * Removes all of the elements from this list.
     */
    public void clear_internal() {
        LinkedList_Entry e/*E=this.L*/ = header.next;
        while (e != header) {
            LinkedList_Entry next/*E=this.L*/ = e.next;
            e.next = e.previous = null;
            e.element = null;
            e = next;
        }
        header.next = header.previous = header;
        size = 0;
        incrementModCount_internal();
    }

	/**
	 * Removes all of the elements from this list.
	 */
	public void clear() {
		synchronized (this.$lock_L) {
			LinkedList_Entry e/*E=this.L*/ = header.next;
			while (e != header) {
			    LinkedList_Entry next/*E=this.L*/ = e.next;
			    e.next = e.previous = null;
			    e.element = null;
			    e = next;
			}
			header.next = header.previous = header;
			size = 0;
			incrementModCount_internal();
		}
	}


    // Positional Access Operations

    /**
     * Returns the element at the specified position in this list.
     *
     * @param index index of element to return.
     * @return the element at the specified position in this list.
     *
     * @throws IndexOutOfBoundsException if the specified index is out of
     * range (<tt>index &lt; 0 || index &gt;= size()</tt>).
     */
    public Object get_internal(int index) {
        return entry_internal(index).element;
    }

	/**
	 * Returns the element at the specified position in this list.
	 * @param index  index of element to return.
	 * @return  the element at the specified position in this list.
	 * @throws IndexOutOfBoundsException  if the specified index is out of range (<tt>index &lt; 0 || index &gt;= size()</tt>).
	 */
	public Object get(int index) {
		synchronized (this.$lock_L) {
			return entry_internal(index).element;
		}
	}

    /**
     * Replaces the element at the specified position in this list with the
     * specified element.
     *
     * @param index index of element to replace.
     * @param element element to be stored at the specified position.
     * @return the element previously at the specified position.
     * @throws IndexOutOfBoundsException if the specified index is out of
     *		  range (<tt>index &lt; 0 || index &gt;= size()</tt>).
     */
    public Object set_internal(int index, Object element) {
        LinkedList_Entry e/*E=this.L*/ = entry_internal(index);
        Object oldVal = e.element;
        e.element = element;
        return oldVal;
    }

	/**
	 * Replaces the element at the specified position in this list with the specified element.
	 * @param index  index of element to replace.
	 * @param element  element to be stored at the specified position.
	 * @return  the element previously at the specified position.
	 * @throws IndexOutOfBoundsException  if the specified index is out of range (<tt>index &lt; 0 || index &gt;= size()</tt>).
	 */
	public Object set(int index, Object element) {
		synchronized (this.$lock_L) {
			LinkedList_Entry e/*E=this.L*/ = entry_internal(index);
			Object oldVal = e.element;
			e.element = element;
			return oldVal;
		}
	}

    /**
     * Inserts the specified element at the specified position in this list.
     * Shifts the element currently at that position (if any) and any
     * subsequent elements to the right (adds one to their indices).
     *
     * @param index index at which the specified element is to be inserted.
     * @param element element to be inserted.
     *
     * @throws IndexOutOfBoundsException if the specified index is out of
     *		  range (<tt>index &lt; 0 || index &gt; size()</tt>).
     */
    public void add_internal(int index, Object element) {
        addBefore_internal(element, (index==size ? header : entry_internal(index)));
    }

	/**
	 * Inserts the specified element at the specified position in this list. Shifts the element currently at that position (if any) and any subsequent elements to the right (adds one to their indices).
	 * @param index  index at which the specified element is to be inserted.
	 * @param element  element to be inserted.
	 * @throws IndexOutOfBoundsException  if the specified index is out of range (<tt>index &lt; 0 || index &gt; size()</tt>).
	 */
	public void add(int index, Object element) {
		synchronized (this.$lock_L) {
			addBefore_internal(element, (index==size ? header : entry_internal(index)));
		}
	}

    /**
     * Removes the element at the specified position in this list.  Shifts any
     * subsequent elements to the left (subtracts one from their indices).
     * Returns the element that was removed from the list.
     *
     * @param index the index of the element to removed.
     * @return the element previously at the specified position.
     *
     * @throws IndexOutOfBoundsException if the specified index is out of
     * 		  range (<tt>index &lt; 0 || index &gt;= size()</tt>).
     */
    public Object remove_internal(int index) {
        return remove_internal(entry_internal(index));
    }

	/**
	 * Removes the element at the specified position in this list.  Shifts any subsequent elements to the left (subtracts one from their indices). Returns the element that was removed from the list.
	 * @param index  the index of the element to removed.
	 * @return  the element previously at the specified position.
	 * @throws IndexOutOfBoundsException  if the specified index is out of range (<tt>index &lt; 0 || index &gt;= size()</tt>).
	 */
	public Object remove(int index) {
		synchronized (this.$lock_L) {
			return remove_internal(entry_internal(index));
		}
	}

    /**
     * Return the indexed entry.
     */
    private LinkedList_Entry/*E=this.L*/ entry_internal(int index) {
        if (index < 0 || index >= size)
            throw new IndexOutOfBoundsException("Index: "+index+
                                                ", Size: "+size);
        LinkedList_Entry e/*E=this.L*/ = header;
        if (index < (size >> 1)) {
            for (int i = 0; i <= index; i++)
                e = e.next;
        } else {
            for (int i = size; i > index; i--)
                e = e.previous;
        }
        return e;
    }

	/**
	 * Return the indexed entry.
	 */
	private LinkedList_Entry entry(int index) {
		synchronized (this.$lock_L) {
			if (index < 0 || index >= size)
			    throw new IndexOutOfBoundsException("Index: "+index+
			                                        ", Size: "+size);
			LinkedList_Entry e/*E=this.L*/ = header;
			if (index < (size >> 1)) {
			    for (int i = 0; i <= index; i++)
			        e = e.next;
			} else {
			    for (int i = size; i > index; i--)
			        e = e.previous;
			}
			return e;
		}
	}


    // Search Operations

    /**
     * Returns the index in this list of the first occurrence of the
     * specified element, or -1 if the List does not contain this
     * element.  More formally, returns the lowest index i such that
     * <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt>, or -1 if
     * there is no such index.
     *
     * @param o element to search for.
     * @return the index in this list of the first occurrence of the
     * 	       specified element, or -1 if the list does not contain this
     * 	       element.
     */
    public int indexOf_internal(Object o) {
        int index = 0;
        if (o==null) {
            for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
                if (e.element==null)
                    return index;
                index++;
            }
        } else {
            for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
                if (o.equals(e.element))
                    return index;
                index++;
            }
        }
        return -1;
    }

	/**
	 * Returns the index in this list of the first occurrence of the specified element, or -1 if the List does not contain this element.  More formally, returns the lowest index i such that <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt>, or -1 if there is no such index.
	 * @param o  element to search for.
	 * @return  the index in this list of the first occurrence of the specified element, or -1 if the list does not contain this element.
	 */
	public int indexOf(Object o) {
		int index = 0;
		synchronized (this.$lock_L) {
			if (o==null) {
			    for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
			        if (e.element==null)
			            return index;
			        index++;
			    }
			} else {
			    for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next) {
			        if (o.equals(e.element))
			            return index;
			        index++;
			    }
			}
		}
		return -1;
	}

    /**
     * Returns the index in this list of the last occurrence of the
     * specified element, or -1 if the list does not contain this
     * element.  More formally, returns the highest index i such that
     * <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt>, or -1 if
     * there is no such index.
     *
     * @param o element to search for.
     * @return the index in this list of the last occurrence of the
     * 	       specified element, or -1 if the list does not contain this
     * 	       element.
     */
    public int lastIndexOf_internal(Object o) {
        int index = size;
        if (o==null) {
            for (LinkedList_Entry e/*E=this.L*/ = header.previous; e != header; e = e.previous) {
                index--;
                if (e.element==null)
                    return index;
            }
        } else {
            for (LinkedList_Entry e/*E=this.L*/ = header.previous; e != header; e = e.previous) {
                index--;
                if (o.equals(e.element))
                    return index;
            }
        }
        return -1;
    }

	/**
	 * Returns the index in this list of the last occurrence of the specified element, or -1 if the list does not contain this element.  More formally, returns the highest index i such that <tt>(o==null ? get(i)==null : o.equals(get(i)))</tt>, or -1 if there is no such index.
	 * @param o  element to search for.
	 * @return  the index in this list of the last occurrence of the specified element, or -1 if the list does not contain this element.
	 */
	public int lastIndexOf(Object o) {
		synchronized (this.$lock_L) {
			int index = size;
			if (o==null) {
			    for (LinkedList_Entry e/*E=this.L*/ = header.previous; e != header; e = e.previous) {
			        index--;
			        if (e.element==null)
			            return index;
			    }
			} else {
			    for (LinkedList_Entry e/*E=this.L*/ = header.previous; e != header; e = e.previous) {
			        index--;
			        if (o.equals(e.element))
			            return index;
			    }
			}
		}
		return -1;
	}

    // Queue operations.

    /**
     * Retrieves, but does not remove, the head (first element) of this list.
     * @return the head of this queue, or <tt>null</tt> if this queue is empty.
     * @since 1.5
     */
    public Object peek_internal() {
        if (size==0)
            return null;
        return getFirst_internal();
    }

	/**
	 * Retrieves, but does not remove, the head (first element) of this list.
	 * @return  the head of this queue, or <tt>null</tt> if this queue is empty.
	 * @since  1.5
	 */
	public Object peek() {
		synchronized (this.$lock_L) {
			if (size==0)
			    return null;
			return getFirst_internal();
		}
	}

    /**
     * Retrieves, but does not remove, the head (first element) of this list.
     * @return the head of this queue.
     * @throws NoSuchElementException if this queue is empty.
     * @since 1.5
     */
    public Object element_internal() {
        return getFirst_internal();
    }

	/**
	 * Retrieves, but does not remove, the head (first element) of this list.
	 * @return  the head of this queue.
	 * @throws NoSuchElementException  if this queue is empty.
	 * @since  1.5
	 */
	public Object element() {
		synchronized (this.$lock_L) {
			return getFirst_internal();
		}
	}

    /**
     * Retrieves and removes the head (first element) of this list.
     * @return the head of this queue, or <tt>null</tt> if this queue is empty.
     * @since 1.5
     */
    public Object poll_internal() {
        if (size==0)
            return null;
        return removeFirst_internal();
    }

	/**
	 * Retrieves and removes the head (first element) of this list.
	 * @return  the head of this queue, or <tt>null</tt> if this queue is empty.
	 * @since  1.5
	 */
	public Object poll() {
		synchronized (this.$lock_L) {
			if (size==0)
			    return null;
			return removeFirst_internal();
		}
	}

    /**
     * Retrieves and removes the head (first element) of this list.
     * @return the head of this queue.
     * @throws NoSuchElementException if this queue is empty.
     * @since 1.5
     */
    public Object remove_internal() {
        return removeFirst_internal();
    }

	/**
	 * Retrieves and removes the head (first element) of this list.
	 * @return  the head of this queue.
	 * @throws NoSuchElementException  if this queue is empty.
	 * @since  1.5
	 */
	public Object remove() {
		synchronized (this.$lock_L) {
			return removeFirst_internal();
		}
	}

    /**
     * Adds the specified element as the tail (last element) of this list.
     *
     * @param o the element to add.
     * @return <tt>true</tt> (as per the general contract of
     * <tt>Queue.offer</tt>)
     * @since 1.5
     */
    public boolean offer_internal(Object o) {
        return add_internal(o);
    }

	/**
	 * Adds the specified element as the tail (last element) of this list.
	 * @param o  the element to add.
	 * @return  <tt>true</tt> (as per the general contract of <tt>Queue.offer</tt>)
	 * @since  1.5
	 */
	public boolean offer(Object o) {
		synchronized (this.$lock_L) {
			return add_internal(o);
		}
	}

    /**
     * Returns a list-iterator of the elements in this list (in proper
     * sequence), starting at the specified position in the list.
     * Obeys the general contract of <tt>List.listIterator(int)</tt>.<p>
     *
     * The list-iterator is <i>fail-fast</i>: if the list is structurally
     * modified at any time after the Iterator is created, in any way except
     * through the list-iterator's own <tt>remove</tt> or <tt>add</tt>
     * methods, the list-iterator will throw a
     * <tt>ConcurrentModificationException</tt>.  Thus, in the face of
     * concurrent modification, the iterator fails quickly and cleanly, rather
     * than risking arbitrary, non-deterministic behavior at an undetermined
     * time in the future.
     *
     * @param index index of first element to be returned from the
     *		    list-iterator (by a call to <tt>next</tt>).
     * @return a ListIterator of the elements in this list (in proper
     * 	       sequence), starting at the specified position in the list.
     * @throws    IndexOutOfBoundsException if index is out of range
     *		  (<tt>index &lt; 0 || index &gt; size()</tt>).
     * @see List#listIterator(int)
     */
    public ListIterator listIterator_internal(int index) {
	return (ListIterator) new LinkedList_ListItr/*I=this.L*/(this, this.header, index, this.$lock_L);
    }

	/**
	 * Returns a list-iterator of the elements in this list (in proper sequence), starting at the specified position in the list. Obeys the general contract of <tt>List.listIterator(int)</tt>.<p> The list-iterator is <i>fail-fast</i>: if the list is structurally modified at any time after the Iterator is created, in any way except through the list-iterator's own <tt>remove</tt> or <tt>add</tt> methods, the list-iterator will throw a <tt>ConcurrentModificationException</tt>.  Thus, in the face of concurrent modification, the iterator fails quickly and cleanly, rather than risking arbitrary, non-deterministic behavior at an undetermined time in the future.
	 * @param index  index of first element to be returned from the list-iterator (by a call to <tt>next</tt>).
	 * @return  a ListIterator of the elements in this list (in proper sequence), starting at the specified position in the list.
	 * @throws IndexOutOfBoundsException  if index is out of range (<tt>index &lt; 0 || index &gt; size()</tt>).
	 * @see List#listIterator(int)
	 */
	public ListIterator listIterator(int index) {
		synchronized (this.$lock_L) {
			return (ListIterator) new LinkedList_ListItr/*I=this.L*/(this, this.header, index, this.$lock_L);
		}
	}

    // FT: made this method private, because it returns an internal class
    private LinkedList_Entry/*E=this.L*/ addBefore_internal(Object o, LinkedList_Entry/*E=this.L*/ e) {
	LinkedList_Entry newEntry/*E=this.L*/ = new LinkedList_Entry/*E=this.L*/(o, e, e.previous, this.$lock_L);
	newEntry.previous.next = newEntry;
	newEntry.next.previous = newEntry;
	size++;
	incrementModCount_internal();
	return newEntry;
    }

	private LinkedList_Entry addBefore(Object o, LinkedList_Entry e) {
		synchronized (this.$lock_L) {
			LinkedList_Entry newEntry/*E=this.L*/ = new LinkedList_Entry/*E=this.L*/(o, e, e.previous, this.$lock_L);
			newEntry.previous.next = newEntry;
			newEntry.next.previous = newEntry;
			size++;
			incrementModCount_internal();
			return newEntry;
		}
	}
    
    // FT: clone of addBefore, in which the return type has been changed to void
    // (this method is called by LinkedList_ListItr.add(), which ignored the return value of addBefore()
    void addBefore2_internal(Object o, LinkedList_Entry/*E=this.L*/ e) {
	LinkedList_Entry newEntry/*E=this.L*/ = new LinkedList_Entry/*E=this.L*/(o, e, e.previous, this.$lock_L);
	newEntry.previous.next = newEntry;
	newEntry.next.previous = newEntry;
	size++;
	incrementModCount_internal();
    }

	void addBefore2(Object o, LinkedList_Entry e) {
		synchronized (this.$lock_L) {
			LinkedList_Entry newEntry/*E=this.L*/ = new LinkedList_Entry/*E=this.L*/(o, e, e.previous, this.$lock_L);
			newEntry.previous.next = newEntry;
			newEntry.next.previous = newEntry;
			size++;
			incrementModCount_internal();
		}
	}

    Object remove_internal(LinkedList_Entry e/*E=this.L*/) {
	if (e == header)
	    throw new NoSuchElementException();

        Object result = e.element;
	e.previous.next = e.next;
	e.next.previous = e.previous;
        e.next = e.previous = null;
        e.element = null;
	size--;
	incrementModCount_internal();
        return result;
    }

	Object remove(LinkedList_Entry e) {
		synchronized (this.$lock_L) {
			if (e == header)
			    throw new NoSuchElementException();
			Object result = e.element;
			e.previous.next = e.next;
			e.next.previous = e.previous;
			e.next = e.previous = null;
			e.element = null;
			size--;
			incrementModCount_internal();
			return result;
		}
	}

    /**
     * Returns a shallow copy of this <tt>LinkedList</tt>. (The elements
     * themselves are not cloned.)
     *
     * @return a shallow copy of this <tt>LinkedList</tt> instance.
     */
// FT: clone() generated automatically by our compiler
//    public Object clone() {
//        LinkedList clone = null;
//	try {
//	    clone = (LinkedList) super.clone();
//	} catch (CloneNotSupportedException e) {
//	    throw new InternalError();
//	}
//
//        // Put clone into "virgin" state
//        clone.header = new LinkedList_Entry/*E=this.L*/(null, null, null);
//        clone.header.next = clone.header.previous = clone.header;
//        clone.size = 0;
//        clone.setModCount(0);
//
//        // Initialize clone with our elements
//        for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next)
//            clone.add(e.element);
//
//        return clone;
//    }

    /**
     * Returns an array containing all of the elements in this list
     * in the correct order.
     *
     * @return an array containing all of the elements in this list
     * 	       in the correct order.
     */
    public Object[] toArray_internal() {
	Object[] result = new Object[size];
        int i = 0;
        for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next)
            result[i++] = e.element;
	return result;
    }

	/**
	 * Returns an array containing all of the elements in this list in the correct order.
	 * @return  an array containing all of the elements in this list in the correct order.
	 */
	public Object[] toArray() {
		synchronized (this.$lock_L) {
			Object[] result = new Object[size];
			int i = 0;
			for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next)
			    result[i++] = e.element;
			return result;
		}
	}

    /**
     * Returns an array containing all of the elements in this list in
     * the correct order; the runtime type of the returned array is that of
     * the specified array.  If the list fits in the specified array, it
     * is returned therein.  Otherwise, a new array is allocated with the
     * runtime type of the specified array and the size of this list.<p>
     *
     * If the list fits in the specified array with room to spare
     * (i.e., the array has more elements than the list),
     * the element in the array immediately following the end of the
     * collection is set to null.  This is useful in determining the length
     * of the list <i>only</i> if the caller knows that the list
     * does not contain any null elements.
     *
     * @param a the array into which the elements of the list are to
     *		be stored, if it is big enough; otherwise, a new array of the
     * 		same runtime type is allocated for this purpose.
     * @return an array containing the elements of the list.
     * @throws ArrayStoreException if the runtime type of a is not a
     *         supertype of the runtime type of every element in this list.
     * @throws NullPointerException if the specified array is null.
     */
    public Object[] toArray_internal(Object[] a) {
        if (a.length < size)
            a = (Object[])java.lang.reflect.Array.newInstance(
                                a.getClass().getComponentType(), size);
        int i = 0;
	Object[] result = a;
        for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next)
            result[i++] = e.element;

        if (a.length > size)
            a[size] = null;

        return a;
    }

	/**
	 * Returns an array containing all of the elements in this list in the correct order; the runtime type of the returned array is that of the specified array.  If the list fits in the specified array, it is returned therein.  Otherwise, a new array is allocated with the runtime type of the specified array and the size of this list.<p> If the list fits in the specified array with room to spare (i.e., the array has more elements than the list), the element in the array immediately following the end of the collection is set to null.  This is useful in determining the length of the list <i>only</i> if the caller knows that the list does not contain any null elements.
	 * @param a  the array into which the elements of the list are to be stored, if it is big enough; otherwise, a new array of the same runtime type is allocated for this purpose.
	 * @return  an array containing the elements of the list.
	 * @throws ArrayStoreException  if the runtime type of a is not a supertype of the runtime type of every element in this list.
	 * @throws NullPointerException  if the specified array is null.
	 */
	public Object[] toArray(Object[] a) {
		synchronized (this.$lock_L) {
			if (a.length < size)
			    a = (Object[])java.lang.reflect.Array.newInstance(
			                        a.getClass().getComponentType(), size);
			int i = 0;
			Object[] result = a;
			for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next)
			    result[i++] = e.element;
			if (a.length > size)
			    a[size] = null;
		}
		return a;
	}

    private static final long serialVersionUID = 876323262645176354L;

    /**
     * Save the state of this <tt>LinkedList</tt> instance to a stream (that
     * is, serialize it).
     *
     * @serialData The size of the list (the number of elements it
     *		   contains) is emitted (int), followed by all of its
     * elements (each an Object) in the proper order.
     */
    private void writeObject_internal(java.io.ObjectOutputStream s)
        throws java.io.IOException {
	// Write out any hidden serialization magic
	s.defaultWriteObject();

        // Write out size
        s.writeInt(size);

	// Write out all elements in the proper order.
        for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next)
            s.writeObject(e.element);
    }

	/**
	 * Save the state of this <tt>LinkedList</tt> instance to a stream (that is, serialize it).
	 * @serialData  The size of the list (the number of elements it contains) is emitted (int), followed by all of its elements (each an Object) in the proper order.
	 */
	private void writeObject(java.io.ObjectOutputStream s)
			throws java.io.IOException {
		synchronized (this.$lock_L) {
			// Write out any hidden serialization magic
			s.defaultWriteObject();
			// Write out size
			s.writeInt(size);
			// Write out all elements in the proper order.
			for (LinkedList_Entry e/*E=this.L*/ = header.next; e != header; e = e.next)
			    s.writeObject(e.element);
		}
	}

    /**
     * Reconstitute this <tt>LinkedList</tt> instance from a stream (that is
     * deserialize it).
     */
    private void readObject_internal(java.io.ObjectInputStream s)
        throws java.io.IOException, ClassNotFoundException {
	// Read in any hidden serialization magic
	s.defaultReadObject();

        // Read in size
        int size = s.readInt();

        // Initialize header
        header = new LinkedList_Entry/*E=this.L*/(null, null, null, this.$lock_L);
        header.next = header.previous = header;

	// Read in all elements in the proper order.
	for (int i=0; i<size; i++)
            addBefore_internal((Object)s.readObject(), header);
    }

	/**
	 * Reconstitute this <tt>LinkedList</tt> instance from a stream (that is deserialize it).
	 */
	private void readObject(java.io.ObjectInputStream s)
			throws java.io.IOException, ClassNotFoundException {
		synchronized (this.$lock_L) {
			// Read in any hidden serialization magic
			s.defaultReadObject();
			// Read in size
			int size = s.readInt();
			// Initialize header
			header = new LinkedList_Entry/*E=this.L*/(null, null, null, this.$lock_L);
			header.next = header.previous = header;
			// Read in all elements in the proper order.
			for (int i=0; i<size; i++)
			        addBefore_internal((Object)s.readObject(), header);
		}
	}
}

